/*
 * translate-rfc3066-private.h - RFC 3066 tag to name mappings
 *
 * Generated automatically by lang2h.sh.
 * Do not edit this file directly.
 */

#ifndef _TRANSLATE_RFC3066_PRIVATE_H
#define _TRANSLATE_RFC3066_PRIVATE_H

static void
translate_rfc3066_init (GHashTable *table)
{
  g_return_if_fail(table != NULL);
  g_hash_table_insert(table, "aa", _("Afar"));
  g_hash_table_insert(table, "ab", _("Abkhazian"));
  g_hash_table_insert(table, "ace", _("Achinese"));
  g_hash_table_insert(table, "ach", _("Acoli"));
  g_hash_table_insert(table, "ada", _("Adangme"));
  g_hash_table_insert(table, "ady", _("Adyghe; Adygei"));
  g_hash_table_insert(table, "afa", _("Afro-Asiatic (Other)"));
  g_hash_table_insert(table, "afh", _("Afrihili"));
  g_hash_table_insert(table, "af", _("Afrikaans"));
  g_hash_table_insert(table, "ak", _("Akan"));
  g_hash_table_insert(table, "akk", _("Akkadian"));
  g_hash_table_insert(table, "sq", _("Albanian"));
  g_hash_table_insert(table, "ale", _("Aleut"));
  g_hash_table_insert(table, "alg", _("Algonquian languages"));
  g_hash_table_insert(table, "am", _("Amharic"));
  g_hash_table_insert(table, "ang", _("English, Old (ca.450-1100)"));
  g_hash_table_insert(table, "apa", _("Apache languages"));
  g_hash_table_insert(table, "ar", _("Arabic"));
  g_hash_table_insert(table, "arc", _("Aramaic"));
  g_hash_table_insert(table, "an", _("Aragonese"));
  g_hash_table_insert(table, "hy", _("Armenian"));
  g_hash_table_insert(table, "arn", _("Araucanian"));
  g_hash_table_insert(table, "arp", _("Arapaho"));
  g_hash_table_insert(table, "art", _("Artificial (Other)"));
  g_hash_table_insert(table, "arw", _("Arawak"));
  g_hash_table_insert(table, "as", _("Assamese"));
  g_hash_table_insert(table, "ast", _("Asturian; Bable"));
  g_hash_table_insert(table, "ath", _("Athapascan languages"));
  g_hash_table_insert(table, "aus", _("Australian languages"));
  g_hash_table_insert(table, "av", _("Avaric"));
  g_hash_table_insert(table, "ae", _("Avestan"));
  g_hash_table_insert(table, "awa", _("Awadhi"));
  g_hash_table_insert(table, "ay", _("Aymara"));
  g_hash_table_insert(table, "az", _("Azerbaijani"));
  g_hash_table_insert(table, "bad", _("Banda"));
  g_hash_table_insert(table, "bai", _("Bamileke languages"));
  g_hash_table_insert(table, "ba", _("Bashkir"));
  g_hash_table_insert(table, "bal", _("Baluchi"));
  g_hash_table_insert(table, "bm", _("Bambara"));
  g_hash_table_insert(table, "ban", _("Balinese"));
  g_hash_table_insert(table, "eu", _("Basque"));
  g_hash_table_insert(table, "bas", _("Basa"));
  g_hash_table_insert(table, "bat", _("Baltic (Other)"));
  g_hash_table_insert(table, "bej", _("Beja"));
  g_hash_table_insert(table, "be", _("Belarusian"));
  g_hash_table_insert(table, "bem", _("Bemba"));
  g_hash_table_insert(table, "bn", _("Bengali"));
  g_hash_table_insert(table, "ber", _("Berber (Other)"));
  g_hash_table_insert(table, "bho", _("Bhojpuri"));
  g_hash_table_insert(table, "bh", _("Bihari"));
  g_hash_table_insert(table, "bik", _("Bikol"));
  g_hash_table_insert(table, "bin", _("Bini"));
  g_hash_table_insert(table, "bi", _("Bislama"));
  g_hash_table_insert(table, "bla", _("Siksika"));
  g_hash_table_insert(table, "bnt", _("Bantu (Other)"));
  g_hash_table_insert(table, "bs", _("Bosnian"));
  g_hash_table_insert(table, "bra", _("Braj"));
  g_hash_table_insert(table, "br", _("Breton"));
  g_hash_table_insert(table, "btk", _("Batak (Indonesia)"));
  g_hash_table_insert(table, "bua", _("Buriat"));
  g_hash_table_insert(table, "bug", _("Buginese"));
  g_hash_table_insert(table, "bg", _("Bulgarian"));
  g_hash_table_insert(table, "my", _("Burmese"));
  g_hash_table_insert(table, "byn", _("Blin; Bilin"));
  g_hash_table_insert(table, "cad", _("Caddo"));
  g_hash_table_insert(table, "cai", _("Central American Indian (Other)"));
  g_hash_table_insert(table, "car", _("Carib"));
  g_hash_table_insert(table, "ca", _("Catalan; Valencian"));
  g_hash_table_insert(table, "cau", _("Caucasian (Other)"));
  g_hash_table_insert(table, "ceb", _("Cebuano"));
  g_hash_table_insert(table, "cel", _("Celtic (Other)"));
  g_hash_table_insert(table, "ch", _("Chamorro"));
  g_hash_table_insert(table, "chb", _("Chibcha"));
  g_hash_table_insert(table, "ce", _("Chechen"));
  g_hash_table_insert(table, "chg", _("Chagatai"));
  g_hash_table_insert(table, "zh", _("Chinese"));
  g_hash_table_insert(table, "chk", _("Chuukese"));
  g_hash_table_insert(table, "chm", _("Mari"));
  g_hash_table_insert(table, "chn", _("Chinook jargon"));
  g_hash_table_insert(table, "cho", _("Choctaw"));
  g_hash_table_insert(table, "chp", _("Chipewyan"));
  g_hash_table_insert(table, "chr", _("Cherokee"));
  g_hash_table_insert(table, "cu", _("Church Slavic; Old Slavonic; Church Slavonic; Old Bulgarian; Old Church Slavonic"));
  g_hash_table_insert(table, "cv", _("Chuvash"));
  g_hash_table_insert(table, "chy", _("Cheyenne"));
  g_hash_table_insert(table, "cmc", _("Chamic languages"));
  g_hash_table_insert(table, "cop", _("Coptic"));
  g_hash_table_insert(table, "kw", _("Cornish"));
  g_hash_table_insert(table, "co", _("Corsican"));
  g_hash_table_insert(table, "cpe", _("Creoles and pidgins, English based (Other)"));
  g_hash_table_insert(table, "cpf", _("Creoles and pidgins, French-based (Other)"));
  g_hash_table_insert(table, "cpp", _("Creoles and pidgins, Portuguese-based (Other)"));
  g_hash_table_insert(table, "cr", _("Cree"));
  g_hash_table_insert(table, "crh", _("Crimean Tatar; Crimean Turkish"));
  g_hash_table_insert(table, "crp", _("Creoles and pidgins (Other)"));
  g_hash_table_insert(table, "csb", _("Kashubian"));
  g_hash_table_insert(table, "cus", _("Cushitic (Other)"));
  g_hash_table_insert(table, "cs", _("Czech"));
  g_hash_table_insert(table, "dak", _("Dakota"));
  g_hash_table_insert(table, "da", _("Danish"));
  g_hash_table_insert(table, "dar", _("Dargwa"));
  g_hash_table_insert(table, "day", _("Dayak"));
  g_hash_table_insert(table, "del", _("Delaware"));
  g_hash_table_insert(table, "den", _("Slave (Athapascan)"));
  g_hash_table_insert(table, "dgr", _("Dogrib"));
  g_hash_table_insert(table, "din", _("Dinka"));
  g_hash_table_insert(table, "dv", _("Divehi"));
  g_hash_table_insert(table, "doi", _("Dogri"));
  g_hash_table_insert(table, "dra", _("Dravidian (Other)"));
  g_hash_table_insert(table, "dsb", _("Lower Sorbian"));
  g_hash_table_insert(table, "dua", _("Duala"));
  g_hash_table_insert(table, "dum", _("Dutch, Middle (ca.1050-1350)"));
  g_hash_table_insert(table, "nl", _("Dutch; Flemish"));
  g_hash_table_insert(table, "dyu", _("Dyula"));
  g_hash_table_insert(table, "dz", _("Dzongkha"));
  g_hash_table_insert(table, "efi", _("Efik"));
  g_hash_table_insert(table, "egy", _("Egyptian (Ancient)"));
  g_hash_table_insert(table, "eka", _("Ekajuk"));
  g_hash_table_insert(table, "elx", _("Elamite"));
  g_hash_table_insert(table, "en", _("English"));
  g_hash_table_insert(table, "enm", _("English, Middle (1100-1500)"));
  g_hash_table_insert(table, "eo", _("Esperanto"));
  g_hash_table_insert(table, "et", _("Estonian"));
  g_hash_table_insert(table, "ee", _("Ewe"));
  g_hash_table_insert(table, "ewo", _("Ewondo"));
  g_hash_table_insert(table, "fan", _("Fang"));
  g_hash_table_insert(table, "fo", _("Faroese"));
  g_hash_table_insert(table, "fat", _("Fanti"));
  g_hash_table_insert(table, "fj", _("Fijian"));
  g_hash_table_insert(table, "fil", _("Filipino; Pilipino"));
  g_hash_table_insert(table, "fi", _("Finnish"));
  g_hash_table_insert(table, "fiu", _("Finno-Ugrian (Other)"));
  g_hash_table_insert(table, "fon", _("Fon"));
  g_hash_table_insert(table, "fr", _("French"));
  g_hash_table_insert(table, "frm", _("French, Middle (ca.1400-1800)"));
  g_hash_table_insert(table, "fro", _("French, Old (842-ca.1400)"));
  g_hash_table_insert(table, "fy", _("Frisian"));
  g_hash_table_insert(table, "ff", _("Fulah"));
  g_hash_table_insert(table, "fur", _("Friulian"));
  g_hash_table_insert(table, "gaa", _("Ga"));
  g_hash_table_insert(table, "gay", _("Gayo"));
  g_hash_table_insert(table, "gba", _("Gbaya"));
  g_hash_table_insert(table, "gem", _("Germanic (Other)"));
  g_hash_table_insert(table, "ka", _("Georgian"));
  g_hash_table_insert(table, "de", _("German"));
  g_hash_table_insert(table, "gez", _("Geez"));
  g_hash_table_insert(table, "gil", _("Gilbertese"));
  g_hash_table_insert(table, "gd", _("Gaelic; Scottish Gaelic"));
  g_hash_table_insert(table, "ga", _("Irish"));
  g_hash_table_insert(table, "gl", _("Gallegan"));
  g_hash_table_insert(table, "gv", _("Manx"));
  g_hash_table_insert(table, "gmh", _("German, Middle High (ca.1050-1500)"));
  g_hash_table_insert(table, "goh", _("German, Old High (ca.750-1050)"));
  g_hash_table_insert(table, "gon", _("Gondi"));
  g_hash_table_insert(table, "gor", _("Gorontalo"));
  g_hash_table_insert(table, "got", _("Gothic"));
  g_hash_table_insert(table, "grb", _("Grebo"));
  g_hash_table_insert(table, "grc", _("Greek, Ancient (to 1453)"));
  g_hash_table_insert(table, "el", _("Greek, Modern (1453-)"));
  g_hash_table_insert(table, "gn", _("Guarani"));
  g_hash_table_insert(table, "gu", _("Gujarati"));
  g_hash_table_insert(table, "gwi", _("Gwich´in"));
  g_hash_table_insert(table, "hai", _("Haida"));
  g_hash_table_insert(table, "ht", _("Haitian; Haitian Creole"));
  g_hash_table_insert(table, "ha", _("Hausa"));
  g_hash_table_insert(table, "haw", _("Hawaiian"));
  g_hash_table_insert(table, "he", _("Hebrew"));
  g_hash_table_insert(table, "hz", _("Herero"));
  g_hash_table_insert(table, "hil", _("Hiligaynon"));
  g_hash_table_insert(table, "him", _("Himachali"));
  g_hash_table_insert(table, "hi", _("Hindi"));
  g_hash_table_insert(table, "hit", _("Hittite"));
  g_hash_table_insert(table, "hmn", _("Hmong"));
  g_hash_table_insert(table, "ho", _("Hiri Motu"));
  g_hash_table_insert(table, "hsb", _("Upper Sorbian"));
  g_hash_table_insert(table, "hu", _("Hungarian"));
  g_hash_table_insert(table, "hup", _("Hupa"));
  g_hash_table_insert(table, "iba", _("Iban"));
  g_hash_table_insert(table, "ig", _("Igbo"));
  g_hash_table_insert(table, "is", _("Icelandic"));
  g_hash_table_insert(table, "io", _("Ido"));
  g_hash_table_insert(table, "ii", _("Sichuan Yi"));
  g_hash_table_insert(table, "ijo", _("Ijo"));
  g_hash_table_insert(table, "iu", _("Inuktitut"));
  g_hash_table_insert(table, "ie", _("Interlingue"));
  g_hash_table_insert(table, "ilo", _("Iloko"));
  g_hash_table_insert(table, "ia", _("Interlingua (International Auxiliary Language Association)"));
  g_hash_table_insert(table, "inc", _("Indic (Other)"));
  g_hash_table_insert(table, "id", _("Indonesian"));
  g_hash_table_insert(table, "ine", _("Indo-European (Other)"));
  g_hash_table_insert(table, "inh", _("Ingush"));
  g_hash_table_insert(table, "ik", _("Inupiaq"));
  g_hash_table_insert(table, "ira", _("Iranian (Other)"));
  g_hash_table_insert(table, "iro", _("Iroquoian languages"));
  g_hash_table_insert(table, "it", _("Italian"));
  g_hash_table_insert(table, "jv", _("Javanese"));
  g_hash_table_insert(table, "jbo", _("Lojban"));
  g_hash_table_insert(table, "ja", _("Japanese"));
  g_hash_table_insert(table, "jpr", _("Judeo-Persian"));
  g_hash_table_insert(table, "jrb", _("Judeo-Arabic"));
  g_hash_table_insert(table, "kaa", _("Kara-Kalpak"));
  g_hash_table_insert(table, "kab", _("Kabyle"));
  g_hash_table_insert(table, "kac", _("Kachin"));
  g_hash_table_insert(table, "kl", _("Kalaallisut; Greenlandic"));
  g_hash_table_insert(table, "kam", _("Kamba"));
  g_hash_table_insert(table, "kn", _("Kannada"));
  g_hash_table_insert(table, "kar", _("Karen"));
  g_hash_table_insert(table, "ks", _("Kashmiri"));
  g_hash_table_insert(table, "kr", _("Kanuri"));
  g_hash_table_insert(table, "kaw", _("Kawi"));
  g_hash_table_insert(table, "kk", _("Kazakh"));
  g_hash_table_insert(table, "kbd", _("Kabardian"));
  g_hash_table_insert(table, "kha", _("Khasi"));
  g_hash_table_insert(table, "khi", _("Khoisan (Other)"));
  g_hash_table_insert(table, "km", _("Khmer"));
  g_hash_table_insert(table, "kho", _("Khotanese"));
  g_hash_table_insert(table, "ki", _("Kikuyu; Gikuyu"));
  g_hash_table_insert(table, "rw", _("Kinyarwanda"));
  g_hash_table_insert(table, "ky", _("Kirghiz"));
  g_hash_table_insert(table, "kmb", _("Kimbundu"));
  g_hash_table_insert(table, "kok", _("Konkani"));
  g_hash_table_insert(table, "kv", _("Komi"));
  g_hash_table_insert(table, "kg", _("Kongo"));
  g_hash_table_insert(table, "ko", _("Korean"));
  g_hash_table_insert(table, "kos", _("Kosraean"));
  g_hash_table_insert(table, "kpe", _("Kpelle"));
  g_hash_table_insert(table, "krc", _("Karachay-Balkar"));
  g_hash_table_insert(table, "kro", _("Kru"));
  g_hash_table_insert(table, "kru", _("Kurukh"));
  g_hash_table_insert(table, "kj", _("Kuanyama; Kwanyama"));
  g_hash_table_insert(table, "kum", _("Kumyk"));
  g_hash_table_insert(table, "ku", _("Kurdish"));
  g_hash_table_insert(table, "kut", _("Kutenai"));
  g_hash_table_insert(table, "lad", _("Ladino"));
  g_hash_table_insert(table, "lah", _("Lahnda"));
  g_hash_table_insert(table, "lam", _("Lamba"));
  g_hash_table_insert(table, "lo", _("Lao"));
  g_hash_table_insert(table, "la", _("Latin"));
  g_hash_table_insert(table, "lv", _("Latvian"));
  g_hash_table_insert(table, "lez", _("Lezghian"));
  g_hash_table_insert(table, "li", _("Limburgan; Limburger; Limburgish"));
  g_hash_table_insert(table, "ln", _("Lingala"));
  g_hash_table_insert(table, "lt", _("Lithuanian"));
  g_hash_table_insert(table, "lol", _("Mongo"));
  g_hash_table_insert(table, "loz", _("Lozi"));
  g_hash_table_insert(table, "lb", _("Luxembourgish; Letzeburgesch"));
  g_hash_table_insert(table, "lua", _("Luba-Lulua"));
  g_hash_table_insert(table, "lu", _("Luba-Katanga"));
  g_hash_table_insert(table, "lg", _("Ganda"));
  g_hash_table_insert(table, "lui", _("Luiseno"));
  g_hash_table_insert(table, "lun", _("Lunda"));
  g_hash_table_insert(table, "luo", _("Luo (Kenya and Tanzania)"));
  g_hash_table_insert(table, "lus", _("lushai"));
  g_hash_table_insert(table, "mk", _("Macedonian"));
  g_hash_table_insert(table, "mad", _("Madurese"));
  g_hash_table_insert(table, "mag", _("Magahi"));
  g_hash_table_insert(table, "mh", _("Marshallese"));
  g_hash_table_insert(table, "mai", _("Maithili"));
  g_hash_table_insert(table, "mak", _("Makasar"));
  g_hash_table_insert(table, "ml", _("Malayalam"));
  g_hash_table_insert(table, "man", _("Mandingo"));
  g_hash_table_insert(table, "mi", _("Maori"));
  g_hash_table_insert(table, "map", _("Austronesian (Other)"));
  g_hash_table_insert(table, "mr", _("Marathi"));
  g_hash_table_insert(table, "mas", _("Masai"));
  g_hash_table_insert(table, "ms", _("Malay"));
  g_hash_table_insert(table, "mdf", _("Moksha"));
  g_hash_table_insert(table, "mdr", _("Mandar"));
  g_hash_table_insert(table, "men", _("Mende"));
  g_hash_table_insert(table, "mga", _("Irish, Middle (900-1200)"));
  g_hash_table_insert(table, "mic", _("Mi'kmaq; Micmac"));
  g_hash_table_insert(table, "min", _("Minangkabau"));
  g_hash_table_insert(table, "mis", _("Miscellaneous languages"));
  g_hash_table_insert(table, "mkh", _("Mon-Khmer (Other)"));
  g_hash_table_insert(table, "mg", _("Malagasy"));
  g_hash_table_insert(table, "mt", _("Maltese"));
  g_hash_table_insert(table, "mnc", _("Manchu"));
  g_hash_table_insert(table, "mni", _("Manipuri"));
  g_hash_table_insert(table, "mno", _("Manobo languages"));
  g_hash_table_insert(table, "moh", _("Mohawk"));
  g_hash_table_insert(table, "mo", _("Moldavian"));
  g_hash_table_insert(table, "mn", _("Mongolian"));
  g_hash_table_insert(table, "mos", _("Mossi"));
  g_hash_table_insert(table, "mul", _("Multiple languages"));
  g_hash_table_insert(table, "mun", _("Munda languages"));
  g_hash_table_insert(table, "mus", _("Creek"));
  g_hash_table_insert(table, "mwl", _("Mirandese"));
  g_hash_table_insert(table, "mwr", _("Marwari"));
  g_hash_table_insert(table, "myn", _("Mayan languages"));
  g_hash_table_insert(table, "myv", _("Erzya"));
  g_hash_table_insert(table, "nah", _("Nahuatl"));
  g_hash_table_insert(table, "nai", _("North American Indian"));
  g_hash_table_insert(table, "nap", _("Neapolitan"));
  g_hash_table_insert(table, "na", _("Nauru"));
  g_hash_table_insert(table, "nv", _("Navajo; Navaho"));
  g_hash_table_insert(table, "nr", _("Ndebele, South; South Ndebele"));
  g_hash_table_insert(table, "nd", _("Ndebele, North; North Ndebele"));
  g_hash_table_insert(table, "ng", _("Ndonga"));
  g_hash_table_insert(table, "nds", _("Low German; Low Saxon; German, Low; Saxon, Low"));
  g_hash_table_insert(table, "ne", _("Nepali"));
  g_hash_table_insert(table, "new", _("Nepal Bhasa; Newari"));
  g_hash_table_insert(table, "nia", _("Nias"));
  g_hash_table_insert(table, "nic", _("Niger-Kordofanian (Other)"));
  g_hash_table_insert(table, "niu", _("Niuean"));
  g_hash_table_insert(table, "nn", _("Norwegian Nynorsk"));
  g_hash_table_insert(table, "nb", _("Norwegian Bokmål"));
  g_hash_table_insert(table, "nog", _("Nogai"));
  g_hash_table_insert(table, "non", _("Norse, Old"));
  g_hash_table_insert(table, "no", _("Norwegian"));
  g_hash_table_insert(table, "nso", _("Northern Sotho; Pedi; Sepedi"));
  g_hash_table_insert(table, "nub", _("Nubian languages"));
  g_hash_table_insert(table, "nwc", _("Classical Newari; Old Newari; Classical Nepal Bhasa"));
  g_hash_table_insert(table, "ny", _("Chichewa; Chewa; Nyanja"));
  g_hash_table_insert(table, "nym", _("Nyamwezi"));
  g_hash_table_insert(table, "nyn", _("Nyankole"));
  g_hash_table_insert(table, "nyo", _("Nyoro"));
  g_hash_table_insert(table, "nzi", _("Nzima"));
  g_hash_table_insert(table, "oc", _("Occitan (post 1500); Provençal"));
  g_hash_table_insert(table, "oj", _("Ojibwa"));
  g_hash_table_insert(table, "or", _("Oriya"));
  g_hash_table_insert(table, "om", _("Oromo"));
  g_hash_table_insert(table, "osa", _("Osage"));
  g_hash_table_insert(table, "os", _("Ossetian; Ossetic"));
  g_hash_table_insert(table, "ota", _("Turkish, Ottoman (1500-1928)"));
  g_hash_table_insert(table, "oto", _("Otomian languages"));
  g_hash_table_insert(table, "paa", _("Papuan (Other)"));
  g_hash_table_insert(table, "pag", _("Pangasinan"));
  g_hash_table_insert(table, "pal", _("Pahlavi"));
  g_hash_table_insert(table, "pam", _("Pampanga"));
  g_hash_table_insert(table, "pa", _("Panjabi; Punjabi"));
  g_hash_table_insert(table, "pap", _("Papiamento"));
  g_hash_table_insert(table, "pau", _("Palauan"));
  g_hash_table_insert(table, "peo", _("Persian, Old (ca.600-400 B.C.)"));
  g_hash_table_insert(table, "fa", _("Persian"));
  g_hash_table_insert(table, "phi", _("Philippine (Other)"));
  g_hash_table_insert(table, "phn", _("Phoenician"));
  g_hash_table_insert(table, "pi", _("Pali"));
  g_hash_table_insert(table, "pl", _("Polish"));
  g_hash_table_insert(table, "pon", _("Pohnpeian"));
  g_hash_table_insert(table, "pt", _("Portuguese"));
  g_hash_table_insert(table, "pra", _("Prakrit languages"));
  g_hash_table_insert(table, "pro", _("Provençal, Old (to 1500)"));
  g_hash_table_insert(table, "ps", _("Pushto"));
  g_hash_table_insert(table, "qaa-qtz ", _("Reserved for local use"));
  g_hash_table_insert(table, "qu", _("Quechua"));
  g_hash_table_insert(table, "raj", _("Rajasthani"));
  g_hash_table_insert(table, "rap", _("Rapanui"));
  g_hash_table_insert(table, "rar", _("Rarotongan"));
  g_hash_table_insert(table, "roa", _("Romance (Other)"));
  g_hash_table_insert(table, "rm", _("Raeto-Romance"));
  g_hash_table_insert(table, "rom", _("Romany"));
  g_hash_table_insert(table, "ro", _("Romanian"));
  g_hash_table_insert(table, "rn", _("Rundi"));
  g_hash_table_insert(table, "ru", _("Russian"));
  g_hash_table_insert(table, "sad", _("Sandawe"));
  g_hash_table_insert(table, "sg", _("Sango"));
  g_hash_table_insert(table, "sah", _("Yakut"));
  g_hash_table_insert(table, "sai", _("South American Indian (Other)"));
  g_hash_table_insert(table, "sal", _("Salishan languages"));
  g_hash_table_insert(table, "sam", _("Samaritan Aramaic"));
  g_hash_table_insert(table, "sa", _("Sanskrit"));
  g_hash_table_insert(table, "sas", _("Sasak"));
  g_hash_table_insert(table, "sat", _("Santali"));
  g_hash_table_insert(table, "sr", _("Serbian"));
  g_hash_table_insert(table, "scn", _("Sicilian"));
  g_hash_table_insert(table, "sco", _("Scots"));
  g_hash_table_insert(table, "hr", _("Croatian"));
  g_hash_table_insert(table, "sel", _("Selkup"));
  g_hash_table_insert(table, "sem", _("Semitic (Other)"));
  g_hash_table_insert(table, "sga", _("Irish, Old (to 900)"));
  g_hash_table_insert(table, "sgn", _("Sign Languages"));
  g_hash_table_insert(table, "shn", _("Shan"));
  g_hash_table_insert(table, "sid", _("Sidamo"));
  g_hash_table_insert(table, "si", _("Sinhala; Sinhalese"));
  g_hash_table_insert(table, "sio", _("Siouan languages"));
  g_hash_table_insert(table, "sit", _("Sino-Tibetan (Other)"));
  g_hash_table_insert(table, "sla", _("Slavic (Other)"));
  g_hash_table_insert(table, "sk", _("Slovak"));
  g_hash_table_insert(table, "sl", _("Slovenian"));
  g_hash_table_insert(table, "sma", _("Southern Sami"));
  g_hash_table_insert(table, "se", _("Northern Sami"));
  g_hash_table_insert(table, "smi", _("Sami languages (Other)"));
  g_hash_table_insert(table, "smj", _("Lule Sami"));
  g_hash_table_insert(table, "smn", _("Inari Sami"));
  g_hash_table_insert(table, "sm", _("Samoan"));
  g_hash_table_insert(table, "sms", _("Skolt Sami"));
  g_hash_table_insert(table, "sn", _("Shona"));
  g_hash_table_insert(table, "sd", _("Sindhi"));
  g_hash_table_insert(table, "snk", _("Soninke"));
  g_hash_table_insert(table, "sog", _("Sogdian"));
  g_hash_table_insert(table, "so", _("Somali"));
  g_hash_table_insert(table, "son", _("Songhai"));
  g_hash_table_insert(table, "st", _("Sotho, Southern"));
  g_hash_table_insert(table, "es", _("Spanish; Castilian"));
  g_hash_table_insert(table, "sc", _("Sardinian"));
  g_hash_table_insert(table, "srr", _("Serer"));
  g_hash_table_insert(table, "ssa", _("Nilo-Saharan (Other)"));
  g_hash_table_insert(table, "ss", _("Swati"));
  g_hash_table_insert(table, "suk", _("Sukuma"));
  g_hash_table_insert(table, "su", _("Sundanese"));
  g_hash_table_insert(table, "sus", _("Susu"));
  g_hash_table_insert(table, "sux", _("Sumerian"));
  g_hash_table_insert(table, "sw", _("Swahili"));
  g_hash_table_insert(table, "sv", _("Swedish"));
  g_hash_table_insert(table, "syr", _("Syriac"));
  g_hash_table_insert(table, "ty", _("Tahitian"));
  g_hash_table_insert(table, "tai", _("Tai (Other)"));
  g_hash_table_insert(table, "ta", _("Tamil"));
  g_hash_table_insert(table, "tt", _("Tatar"));
  g_hash_table_insert(table, "te", _("Telugu"));
  g_hash_table_insert(table, "tem", _("Timne"));
  g_hash_table_insert(table, "ter", _("Tereno"));
  g_hash_table_insert(table, "tet", _("Tetum"));
  g_hash_table_insert(table, "tg", _("Tajik"));
  g_hash_table_insert(table, "tl", _("Tagalog"));
  g_hash_table_insert(table, "th", _("Thai"));
  g_hash_table_insert(table, "bo", _("Tibetan"));
  g_hash_table_insert(table, "tig", _("Tigre"));
  g_hash_table_insert(table, "ti", _("Tigrinya"));
  g_hash_table_insert(table, "tiv", _("Tiv"));
  g_hash_table_insert(table, "tkl", _("Tokelau"));
  g_hash_table_insert(table, "tlh", _("Klingon; tlhIngan-Hol"));
  g_hash_table_insert(table, "tli", _("Tlingit"));
  g_hash_table_insert(table, "tmh", _("Tamashek"));
  g_hash_table_insert(table, "tog", _("Tonga (Nyasa)"));
  g_hash_table_insert(table, "to", _("Tonga (Tonga Islands)"));
  g_hash_table_insert(table, "tpi", _("Tok Pisin"));
  g_hash_table_insert(table, "tsi", _("Tsimshian"));
  g_hash_table_insert(table, "tn", _("Tswana"));
  g_hash_table_insert(table, "ts", _("Tsonga"));
  g_hash_table_insert(table, "tk", _("Turkmen"));
  g_hash_table_insert(table, "tum", _("Tumbuka"));
  g_hash_table_insert(table, "tup", _("Tupi languages"));
  g_hash_table_insert(table, "tr", _("Turkish"));
  g_hash_table_insert(table, "tut", _("Altaic (Other)"));
  g_hash_table_insert(table, "tvl", _("Tuvalu"));
  g_hash_table_insert(table, "tw", _("Twi"));
  g_hash_table_insert(table, "tyv", _("Tuvinian"));
  g_hash_table_insert(table, "udm", _("Udmurt"));
  g_hash_table_insert(table, "uga", _("Ugaritic"));
  g_hash_table_insert(table, "ug", _("Uighur; Uyghur"));
  g_hash_table_insert(table, "uk", _("Ukrainian"));
  g_hash_table_insert(table, "umb", _("Umbundu"));
  g_hash_table_insert(table, "und", _("Undetermined"));
  g_hash_table_insert(table, "ur", _("Urdu"));
  g_hash_table_insert(table, "uz", _("Uzbek"));
  g_hash_table_insert(table, "vai", _("Vai"));
  g_hash_table_insert(table, "ve", _("Venda"));
  g_hash_table_insert(table, "vi", _("Vietnamese"));
  g_hash_table_insert(table, "vo", _("Volapük"));
  g_hash_table_insert(table, "vot", _("Votic"));
  g_hash_table_insert(table, "wak", _("Wakashan languages"));
  g_hash_table_insert(table, "wal", _("Walamo"));
  g_hash_table_insert(table, "war", _("Waray"));
  g_hash_table_insert(table, "was", _("Washo"));
  g_hash_table_insert(table, "cy", _("Welsh"));
  g_hash_table_insert(table, "wen", _("Sorbian languages"));
  g_hash_table_insert(table, "wa", _("Walloon"));
  g_hash_table_insert(table, "wo", _("Wolof"));
  g_hash_table_insert(table, "xal", _("Kalmyk"));
  g_hash_table_insert(table, "xh", _("Xhosa"));
  g_hash_table_insert(table, "yao", _("Yao"));
  g_hash_table_insert(table, "yap", _("Yapese"));
  g_hash_table_insert(table, "yi", _("Yiddish"));
  g_hash_table_insert(table, "yo", _("Yoruba"));
  g_hash_table_insert(table, "ypk", _("Yupik languages"));
  g_hash_table_insert(table, "zap", _("Zapotec"));
  g_hash_table_insert(table, "zen", _("Zenaga"));
  g_hash_table_insert(table, "za", _("Zhuang; Chuang"));
  g_hash_table_insert(table, "znd", _("Zande"));
  g_hash_table_insert(table, "zu", _("Zulu"));
  g_hash_table_insert(table, "zun", _("Zuni"));
}

#endif /* _TRANSLATE_RFC3066_PRIVATE_H */
